
import java.util.Properties;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NameClassPair;


public class JNDIView
{
   boolean verbose;

   public JNDIView (String[] _args) throws Exception
   {
      Properties p = new Properties ();
      verbose = false;
      InputStream in = getClass ().getResourceAsStream ("/default.properties");

      // process parameters
      int pos = 0;
      while (pos < _args.length)
      {
         if (_args[pos].equals ("-v"))
            verbose = true;
         else if (_args[pos].startsWith ("-h") || _args[pos].equals ("-?"))
         {
            System.out.println ("usage: "+getClass ().getName ()+" [-v] [-h] [jndi.properties file]");
            System.out.println ("       -v ... verbose: list additional the types behind the names");
            System.out.println ("       -h ... help: print this help");
            System.out.println ("       jndi.properties file enables you to specify a jndi.properties file");
            System.out.println ("                       if not given, the default.properties is used");
            System.out.println ("");
            System.out.println ("C-2000 Daniel Schulze <daniel.schulze@telkel.com>");
            System.exit (0);
         }
         else
         {
            // must be a alternative jndi.properties file
            try
            {
               in = new FileInputStream (_args[pos]);
            }
            catch (Exception _e) // FileNotFoundException, SecurityException
            {
               System.out.println ("No access to "+_args[pos]+". exception: "+_e.getMessage ());
               System.out.println ("using default settings...");
            }
         }
         ++pos;
      }

      // load jndi properties
      try
      {
         p.load (in);
      }
      catch (Exception _e) // IOException
      {
         System.out.println ("unable to read properties... ");
         System.exit (1);
      }

      // and go
      try
      {
         Context ctx = new InitialContext (p);
         System.out.println ("InitialContext");
         listContext (ctx, " ");
      }
      catch (Exception _e)
      {
         System.out.println ("no initial context available: " + _e.getMessage ());
         System.exit (0);
      }
   }


   private void listContext (Context _ctx, String _indent)
   {
      try
      {
         // list this context
         NamingEnumeration ne = _ctx.list ("");
         while (ne.hasMore ())
         {
            NameClassPair pair = (NameClassPair)ne.next ();

            // is context ?
            boolean recursive = false;
            try {
               Class c = Class.forName (pair.getClassName ());
               if (Context.class.isAssignableFrom (c))
                  recursive = true;
            } catch (ClassNotFoundException _cnfe) {}

            System.out.println (_indent +  " +- " + pair.getName () +
                                (verbose ? (" (class: "+pair.getClassName ()+")") : ""));
            if (recursive)
               listContext ((Context)_ctx.lookup (pair.getName ()), _indent + (ne.hasMore () ? " |  " : "    "));
         }
         ne.close ();
      }
      catch (NamingException _ne)
      {
         System.out.println ("error while listing context "+_ctx.toString () + ": " + _ne.getMessage ());
      }
   }


   public static void main (String[] _args) throws Exception
   {
      new JNDIView (_args);
   }

}