package ckorn.studienarbeit.bibliothek.bean;

import javax.ejb.*;
import java.rmi.*;
import javax.rmi.*; 
import java.util.*;
import javax.naming.*;
import ckorn.studienarbeit.buch.interfaces.*;

/** Die eigentliche Bibliotheks-Bean */
public class BibliothekBean implements SessionBean {
	private static final String JNDI_BUCH="bibliothek/Buch"; //der JNDI-Name der Buch-Bean

	/** Fgt ein Buch in die Bibliothek ein
	 *	@param title der Buchtitel
	 *	@param invNo die Inventarnummer
	 *	@param author der Buchauthor
	 *	@param pubFirm der Verlag
	*/
	public void addBook(int invNo, String title, String author, String pubFirm) throws DuplicateKeyException, CreateException {
		try {
			// Home-Interface holen und mit dessen Hilfe eine neue Buch-Bean erstellen
			BuchHome buchHome = this.getBuchHome();
			BuchRemote buch = buchHome.create((new InvNo(invNo)), title, author, pubFirm);		
		}
		catch(RemoteException e) {
			System.out.println("RMI-Fehler!");			
		}
	}

	/** Lscht ein Buch aus der Bibliothek
	 *	@param invNo Inventarnummer des zu lschenden Buches
	*/
	public void removeBook(int invNo) throws RemoveException {
		try {
			// Mit Hilfe des Home-Interfaces ein Buch ber die Inventarnummer lschen
			BuchHome buchHome = this.getBuchHome();
			buchHome.remove(new InvNo(invNo));
		}
		catch(RemoteException e) {
			System.out.println("RMI-Fehler!");			
		}
	}

	/** Sucht ein Buch nach ber die Inventarnummer
	 *	@param invNo die Inventarnummer, des Buches, nach dem gesucht wird
	 *	@return das Remote-Interface des gesuchten Buches 
	*/
	public BuchRemote getBookByInvNo(int invNo) throws FinderException {
		BuchRemote buchRemote = null;
		try {
			// Buch ber Home-Interface nach Primrschlssel suchen
			BuchHome buchHome = this.getBuchHome();
			buchRemote = buchHome.findByPrimaryKey(new InvNo(invNo));
		}
		catch(RemoteException e) {
			System.out.println("RMI-Fehler!");			
		}
		return buchRemote;
	}	

	/** Gibt alle Bcher in der Bibliothek zurck
	 *	@return eine Collection aller Bcher
	*/	
	public Collection getAllBooks() {
		Collection buecher = new Vector();
		try {
			// alle Bcher ber Home-Interface suchen
			BuchHome buchHome = this.getBuchHome();
			buecher = buchHome.findAll();		
		}
		catch(RemoteException e) {
			System.out.println("RMI-Fehler!");			
		}
		catch(FinderException e) {
			System.out.println("Warnung: Keine Buecher gefunden!");			
		}		
		return buecher;
	}

	/** Sucht alle Bcher eines Autors
	 *	@param author der Author, dessen Bcher gesucht werden sollen
	 *	@return eine Collection aller Bcher des Authors
	*/
	public Collection getAuthorsBooks(String author) {
		Collection buecher = new Vector();
		try {
			// Alle Bcher eines Autors ber Home-Interface suchen
			BuchHome buchHome = this.getBuchHome();
			buecher = buchHome.findByAuthor(author);		
		}
		catch(RemoteException e) {
			System.out.println("RMI-Fehler!");			
		}
		catch(FinderException e) {
			System.out.println("Warnung: Keine Buecher von Autor "+author+" gefunden!");			
		}		
		return buecher;
	}

	/** Zhlt die Bcher in der Bibliothek
	 *	@return die Anzahl der Bcher
	*/
	public int countBooks() {
		// Alle Bcher als Collection geben lassen und Groesse der Collection bestimmen
		return (this.getAllBooks().size());
	}

	/** gibt das Home-Interfaces einer Buch-Bean zurck
	 * @return das Home-Interface des Buches
	 */
	private BuchHome getBuchHome() {
		BuchHome buchHome = null;
		try {
			// Versuchen das Home-Interface ber das JNDI zu erfragen und Typecast
			InitialContext context = new InitialContext();
			Object remObj = context.lookup(this.JNDI_BUCH);
			buchHome = ((BuchHome) PortableRemoteObject.narrow(remObj, BuchHome.class));	
		}
		catch(NamingException e) {
			System.out.println("Fehler: Ressource "+JNDI_BUCH+" konnte nicht gefunden werden!");
		}
		return(buchHome);
	}

	/* leere Methodenrmpfe -> zustandslose Sessionbean */
	public BibliothekBean() {}
	public void ejbCreate() {}
	public void ejbRemove() {}
	public void ejbActivate() {}
	public void ejbPassivate() {}
	public void setSessionContext(SessionContext sc) {}
}  
