package ckorn.studienarbeit.bibliothek.interfaces;

import javax.ejb.*;
import java.rmi.*;
import java.util.*;
import ckorn.studienarbeit.buch.interfaces.*;

/** Remote-Interface der zustandslossen Sessionbean Bibliotheks-Bean */
public interface BibliothekRemote extends EJBObject {
	/** Fgt ein Buch in die Bibliothek ein
	 *	@param title der Buchtitel
	 *	@param invNo die Inventarnummer
	 *	@param author der Buchauthor
	 *	@param pubFirm der Verlag
	*/
	public void addBook(int invNo, String title, String author, String pubFirm) throws RemoteException, DuplicateKeyException, CreateException;
	
	/** Gibt alle Bcher in der Bibliothek zurck
	 *	@return eine Collection aller Bcher
	*/
	public Collection getAllBooks() throws RemoteException; 
	
	/** Lscht ein Buch aus der Bibliothek
	 *	@param invNo Inventarnummer des zu lschenden Buches
	*/
	public void removeBook(int invNo) throws RemoteException, RemoveException;
	
	/** Sucht alle Bcher eines Autors
	 *	@param author der Autor, dessen Bcher gesucht werden sollen
	 *	@return eine Collection aller Bcher des Autors
	*/
	public Collection getAuthorsBooks(String author) throws RemoteException;
	
	/** Zhlt die Bcher in der Bibliothek
	 *	@return die Anzahl der Bcher
	*/
	public int countBooks() throws RemoteException;
	
	/** Sucht ein Buch nach ber die Inventarnummer
	 *	@param invNo die Inventarnummer, des Buches, nach dem gesucht wird
	 *	@return das Remote-Interface des gesuchten Buches 
	*/
	public BuchRemote getBookByInvNo(int invNo) throws RemoteException, FinderException;
}
